unit modules.login;

{$mode objfpc}{$H+}

// Define dbgWrite to be commented in release mode
{$Macro on}
{$IfDef Debug}
  {$Define dbgWrite:=WriteLn}
{$Else}
  {$Define dbgWrite:=//WriteLn}
{$EndIf}

interface

uses
  SysUtils, Classes,
  // http shit
  httpdefs, fpHTTP, fpWeb,
  // connection
  Connection, Connection.Types, Connection.login,
  // modules
  modules.util;

type

  { TLoginModule }

  TLoginModule = class(TFPWebModule)
    procedure ChangePasswordRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: boolean);
    procedure CreateAccountRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: boolean);
    procedure DataModuleCreate(Sender: TObject);
    procedure LoginRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: boolean);
    procedure LogoutRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: boolean);
    procedure PasswordHintRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: boolean);
  private
    FDBInfo: TDBInfo;
  public

  end;

var
  LoginModule: TLoginModule;

implementation

{$R *.lfm}

{ TLoginModule }

procedure TLoginModule.ChangePasswordRequest(Sender: TObject;
  ARequest: TRequest; AResponse: TResponse; var Handled: boolean);
var
  LoginConnection: TLoginConnection;
begin
  WriteLn('Change password request from: ', ARequest.RemoteAddress);
  // create connection
  LoginConnection := TLoginConnection.Create(ARequest.RemoteAddress);
  try
    try
      // connect to database
      LoginConnection.Connect(FDBInfo);
      // perform password change
      LoginConnection.ChangePassword(
        ARequest.ContentFields.Values['OPass'],
        ARequest.ContentFields.Values['NPass'],
        ARequest.ContentFields.Values['Hint'],
        ARequest.ContentFields.Values['Token']
        );
      // respond with current token
      AResponse.Content := 'Token=' + ARequest.ContentFields.Values['Token'];
    except
      // error occured
      on e: Exception do
      begin
        // pass to client
        AResponse.Content := 'ERR=' + e.Message;
      end;
    end;
  finally
    // cleanup
    LoginConnection.Free;
    // request served
    Handled := True;
  end;
end;

procedure TLoginModule.CreateAccountRequest(Sender: TObject;
  ARequest: TRequest; AResponse: TResponse; var Handled: boolean);
var
  LoginConnection: TLoginConnection;
  tok: TLoginToken;
begin
  dbgWrite('Create account request from: ', ARequest.RemoteAddress);
  // create connection
  LoginConnection := TLoginConnection.Create(ARequest.RemoteAddress);
  try
    try
      // connect to database
      LoginConnection.Connect(FDBInfo);
      // perform account creation
      tok := LoginConnection.RegisterAccount(
        ARequest.ContentFields.Values['EMail'],
        ARequest.ContentFields.Values['Pass'],
        ARequest.ContentFields.Values['Hint']);
      // respond with session token
      AResponse.Content := 'Token=' + tok;
    except
      // error occured
      on e: Exception do
      begin
        // pass to client
        AResponse.Content := 'ERR=' + e.Message;
      end;
    end;
  finally
    // cleanup
    LoginConnection.Free;
    // request served
    Handled := True;
  end;
end;

procedure TLoginModule.DataModuleCreate(Sender: TObject);
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  try
    // load database configuration
    sl.LoadFromFile(SConfigPath + 'db.conf');
    // read config
    with FDBInfo do
    begin
      Hostname := sl.Values['Host'];
      Username := sl.Values['User'];
      Password := sl.Values['Pass'];
      Database := sl.Values['DB'];
    end;
  finally
    sl.Free;
  end;
end;

procedure TLoginModule.LoginRequest(Sender: TObject; ARequest: TRequest;
  AResponse: TResponse; var Handled: boolean);
var
  LoginConnection: TLoginConnection;
  tok: TLoginToken;
begin
  dbgWrite('Loginrequest from: ', ARequest.RemoteAddress);
  // create connection
  LoginConnection := TLoginConnection.Create(ARequest.RemoteAddress);
  try
    try
      // connect to database
      LoginConnection.Connect(FDBInfo);
      // perform login
      tok := LoginConnection.Login(ARequest.ContentFields.Values['EMail'],
        ARequest.ContentFields.Values['Pass']);
      // respond with session token
      AResponse.Content := 'Token=' + tok;
    except
      // error occured
      on e: Exception do
      begin
        // pass to client
        AResponse.Content := 'ERR=' + e.Message;
      end;
    end;
  finally
    // cleanup
    LoginConnection.Free;
    // request served
    Handled := True;
  end;
end;

procedure TLoginModule.LogoutRequest(Sender: TObject; ARequest: TRequest;
  AResponse: TResponse; var Handled: boolean);
var
  LoginConnection: TLoginConnection;
begin
  dbgWrite('logout request from: ', ARequest.RemoteAddress);
  // create connection
  LoginConnection := TLoginConnection.Create(ARequest.RemoteAddress);
  try
    try
      // connect to database
      LoginConnection.Connect(FDBInfo);
      // perform logout
      LoginConnection.LogOut(ARequest.ContentFields.Values['Token']);
      // some status response to signal no error occured
      AResponse.Content := 'Status=ok';
    except
      // some error occured
      on e: Exception do
      begin
        // pass to client
        AResponse.Content := 'ERR=' + e.Message;
      end;
    end;
  finally
    // cleanup
    LoginConnection.Free;
    // request served
    Handled := True;
  end;
end;

procedure TLoginModule.PasswordHintRequest(Sender: TObject; ARequest: TRequest;
  AResponse: TResponse; var Handled: boolean);
var
  LoginConnection: TLoginConnection;
  hint: string;
begin
  dbgWrite('password hint request from: ', ARequest.RemoteAddress);
  // create connection
  LoginConnection := TLoginConnection.Create(ARequest.RemoteAddress);
  try
    try
      // connect to database
      LoginConnection.Connect(FDBInfo);
      // get password hint
      hint := LoginConnection.GetPasswordHint(ARequest.ContentFields.Values['EMail']);
      // respond password hint
      AResponse.Content := 'Hint=' + hint;
    except
      // error occured
      on e: Exception do
      begin
        // pass to client
        AResponse.Content := 'ERR=' + e.Message;
      end;
    end;
  finally
    LoginConnection.Free;
    Handled := True;
  end;
end;

initialization
  RegisterHTTPModule('login', TLoginModule);
end.
