unit modules.userdata;

{$mode objfpc}{$H+}

// Define dbgWrite to be commented in release mode
{$Macro on}
{$IfDef Debug}
  {$Define dbgWrite:=WriteLn}
{$Else}
  {$Define dbgWrite:=//WriteLn}
{$EndIf}

interface

uses
  SysUtils, Classes,
  // http shit
  httpdefs, fpHTTP, fpWeb,
  // connection
  Connection, Connection.Types, Connection.userdata,
  // modules
  modules.util;

type

  { TUserdataModule }

  TUserdataModule = class(TFPWebModule)
    procedure DataModuleCreate(Sender: TObject);
    procedure GetDataRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: boolean);
    procedure SetDataRequest(Sender: TObject; ARequest: TRequest;
      AResponse: TResponse; var Handled: boolean);
  private
    FDBInfo: TDBInfo;
  public
    { public declarations }
  end;

var
  UserdataModule: TUserdataModule;

implementation

{$R *.lfm}

{ TUserdataModule }

procedure TUserdataModule.DataModuleCreate(Sender: TObject);
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  try
    // load database configuration
    sl.LoadFromFile(SConfigPath + 'db.conf');
    // read config
    with FDBInfo do
    begin
      Hostname := sl.Values['Host'];
      Username := sl.Values['User'];
      Password := sl.Values['Pass'];
      Database := sl.Values['DB'];
    end;
  finally
    sl.Free;
  end;
end;

procedure TUserdataModule.GetDataRequest(Sender: TObject; ARequest: TRequest;
  AResponse: TResponse; var Handled: boolean);
var
  UserdataConnection: TUserdataConnection;
begin
  dbgWrite('userdata get request from: ', ARequest.RemoteAddress);
  // create connection
  UserdataConnection := TUserdataConnection.Create(ARequest.RemoteAddr,
    ARequest.ContentFields.Values['Token']);
  try
    try
      // connect to database
      UserdataConnection.Connect(FDBInfo);
      // Clear response content
      AResponse.Contents.Clear;
      // get data from db
      with UserdataConnection.GetUserData do
      begin
        // build response
        AResponse.Contents.Values['ID'] := IntToStr(UserdataConnection.SessionUser);
        AResponse.Contents.Values['Status'] := IntToStr(Status);
        AResponse.Contents.Values['EMail'] := EMail;
        AResponse.Contents.Values['Username'] := UserName;
        AResponse.Contents.Values['FirstName'] := FirstName;
        AResponse.Contents.Values['LastName'] := LastName;
        AResponse.Contents.Values['Phone'] := PhoneNumber;
        AResponse.Contents.Values['Birthday'] := DateToStr(BirthDay);
        AResponse.Contents.Values['Street'] := Adress.Street;
        AResponse.Contents.Values['Housenumber'] := Adress.Number;
        AResponse.Contents.Values['Zipcode'] := Adress.ZipCode;
        AResponse.Contents.Values['City'] := Adress.City;
        AResponse.Contents.Values['AdditionalAdress'] := Adress.AdditionalInformation;
      end;
    except
      // error occured
      on e: Exception do
      begin
        // pass to client
        AResponse.Content := 'ERR=' + e.Message;
      end;
    end;
  finally
    Handled := True;
  end;
end;

procedure TUserdataModule.SetDataRequest(Sender: TObject; ARequest: TRequest;
  AResponse: TResponse; var Handled: boolean);
var
  UserdataConnection: TUserdataConnection;
  Userdata: TUserdata;
begin
  dbgWrite('userdata update request from: ', ARequest.RemoteAddress);
  // create connection
  UserdataConnection := TUserdataConnection.Create(ARequest.RemoteAddr,
    ARequest.ContentFields.Values['Token']);
  try
    try
      // connect to database
      UserdataConnection.Connect(FDBInfo);
      // build Userdata structure
      with Userdata do
      begin
        UserName := ARequest.ContentFields.Values['Username'];
        FirstName := ARequest.ContentFields.Values['FirstName'];
        LastName := ARequest.ContentFields.Values['LastName'];
        PhoneNumber := ARequest.ContentFields.Values['Phone'];
        BirthDay := StrToDate(ARequest.ContentFields.Values['Birthday']);
        Adress.Street := ARequest.ContentFields.Values['Street'];
        Adress.Number := ARequest.ContentFields.Values['Housenumber'];
        Adress.ZipCode := ARequest.ContentFields.Values['Zipcode'];
        Adress.City := ARequest.ContentFields.Values['City'];
        Adress.AdditionalInformation := ARequest.ContentFields.Values['AdditionalAdress'];
      end;
      // write to database
      UserdataConnection.SetUserData(Userdata);
      // Write response
      AResponse.Content := 'Status=ok';
    except
      // error occured
      on e: Exception do
      begin
        // pass to client
        AResponse.Content := 'ERR=' + e.Message;
      end;
    end;
  finally
    Handled := True;
  end;
end;

initialization
  RegisterHTTPModule('userdata', TUserdataModule);
end.

