unit connection.userdata;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  // MySQL units
  mysql56conn, sqldb, DB,
  // cryptography units
  openssl,
  // project units
  Connection, Connection.Types;

type

  { TUserdataConnection }

  TUserdataConnection = class(TSessionConnection)
  public
    function GetUserData: TUserData;
    procedure SetUserData(const AData: TUserData);
  end;

implementation

{ TUserdataConnection }

function TUserdataConnection.GetUserData: TUserData;
begin
  try
    // setup query to retrieve information
    FMySQLQuery.SQL.Text :=
      'SELECT * FROM Accounts JOIN Users ON ID=UID WHERE ID = :uid';
    // fill in params
    FMySQLQuery.ParamByName('uid').AsLargeInt := SessionUser;
    // execute query and get results
    FMySQLQuery.Open;
    // check if viable results
    if FMySQLQuery.RecordCount <> 1 then
      raise EThisShouldNeverHappenException.Create('Recordcount=' +
        IntToStr(FMySQLQuery.RecordCount) + ', this should never happen');
    // fill in return data
    with Result do
    begin
      EMail := FMySQLQuery.FieldByName('EMail').AsString;
      Status := FMySQLQuery.FieldByName('Status').AsInteger;
      UserName := FMySQLQuery.FieldByName('Username').AsString;
      LastName := FMySQLQuery.FieldByName('LastName').AsString;
      BirthDay := FMySQLQuery.FieldByName('Birthday').AsDateTime;
      FirstName := FMySQLQuery.FieldByName('FirstName').AsString;
      PhoneNumber := FMySQLQuery.FieldByName('Phone').AsString;
      Adress.City := FMySQLQuery.FieldByName('City').AsString;
      Adress.Street := FMySQLQuery.FieldByName('Street').AsString;
      Adress.Number := FMySQLQuery.FieldByName('Housenumber').AsString;
      Adress.ZipCode := FMySQLQuery.FieldByName('Zipcode').AsString;
      Adress.AdditionalInformation :=
        FMySQLQuery.FieldByName('AdditionalAddr').AsString;
    end;

  finally
    FMySQLQuery.Close;
    FMySQLTransaction.Commit;
  end;
end;

procedure TUserdataConnection.SetUserData(const AData: TUserData);
begin
  try
    // setup update statement
    FMySQLQuery.SQL.Text :=
      'UPDATE Users SET Username=:uname, FirstName=:fname, ' +
      'LastName=:lname, Birthday=:bday, Phone=:phone, Street=:street,' +
      ' Housenumber=:num, Zipcode=:zip, City=:city, ' +
      'AdditionalAddr=:add WHERE UID=:uid';
    // fill in params
    with AData do
    begin
      FMySQLQuery.ParamByName('uname').AsString := UserName;
      FMySQLQuery.ParamByName('fname').AsString := FirstName;
      FMySQLQuery.ParamByName('lname').AsString := LastName;
      FMySQLQuery.ParamByName('bday').AsDateTime := BirthDay;
      FMySQLQuery.ParamByName('phone').AsString := PhoneNumber;
      FMySQLQuery.ParamByName('street').AsString := Adress.Street;
      FMySQLQuery.ParamByName('num').AsString := Adress.Number;
      FMySQLQuery.ParamByName('zip').AsString := Adress.ZipCode;
      FMySQLQuery.ParamByName('city').AsString := Adress.City;
      FMySQLQuery.ParamByName('add').AsString := Adress.AdditionalInformation;
      FMySQLQuery.ParamByName('uid').AsLargeInt := SessionUser;
    end;
    // execute SQL
    FMySQLQuery.ExecSQL;
  finally
    FMySQLTransaction.Commit;
  end;
end;

end.

